"use strict";
/*
 * Copyright (c) 2023 Huawei Device Co., Ltd.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleReservedConfig = exports.isMatchWildcard = exports.needToBeReserved = exports.wildcardTransformer = exports.containWildcards = exports.separateUniversalReservedItem = exports.isSuperCallStatement = exports.isCommentedNode = exports.collectIdentifiersAndStructs = exports.collectExistNames = void 0;
const typescript_1 = require("typescript");
/**
 * collect exist identifier names in current source file
 * @param sourceFile
 */
function collectExistNames(sourceFile) {
    const identifiers = new Set();
    let visit = (node) => {
        if ((0, typescript_1.isIdentifier)(node)) {
            identifiers.add(node.text);
        }
        (0, typescript_1.forEachChild)(node, visit);
    };
    (0, typescript_1.forEachChild)(sourceFile, visit);
    return identifiers;
}
exports.collectExistNames = collectExistNames;
/**
 * collect exist identifiers in current source file
 * @param sourceFile
 * @param context
 */
function collectIdentifiersAndStructs(sourceFile, context) {
    const identifiers = [];
    const structs = [];
    let visit = (node) => {
        if ((0, typescript_1.isStructDeclaration)(node)) {
            structs.push(node);
        }
        // @ts-ignore
        if (getOriginalNode(node).virtual) {
            return node;
        }
        if (!(0, typescript_1.isIdentifier)(node) || !node.parent) {
            return (0, typescript_1.visitEachChild)(node, visit, context);
        }
        identifiers.push(node);
        return node;
    };
    visit(sourceFile);
    return { shadowIdentifiers: identifiers, shadowStructs: structs };
}
exports.collectIdentifiersAndStructs = collectIdentifiersAndStructs;
function isCommentedNode(node, sourceFile) {
    const ranges = (0, typescript_1.getLeadingCommentRangesOfNode)(node, sourceFile);
    return ranges !== undefined;
}
exports.isCommentedNode = isCommentedNode;
function isSuperCallStatement(node) {
    return (0, typescript_1.isExpressionStatement)(node) &&
        (0, typescript_1.isCallExpression)(node.expression) &&
        node.expression.expression.kind === typescript_1.SyntaxKind.SuperKeyword;
}
exports.isSuperCallStatement = isSuperCallStatement;
/**
 * separate wildcards from specific items.
 */
function separateUniversalReservedItem(originalArray) {
    if (!originalArray) {
        throw new Error('Unable to handle the empty array.');
    }
    const reservedInfo = {
        universalReservedArray: [],
        specificReservedArray: []
    };
    originalArray.forEach(reservedItem => {
        if (containWildcards(reservedItem)) {
            const regexPattern = wildcardTransformer(reservedItem);
            const regexOperator = new RegExp(`^${regexPattern}$`);
            reservedInfo.universalReservedArray.push(regexOperator);
        }
        else {
            reservedInfo.specificReservedArray.push(reservedItem);
        }
    });
    return reservedInfo;
}
exports.separateUniversalReservedItem = separateUniversalReservedItem;
/**
 * check if the item contains '*', '?'.
 */
function containWildcards(item) {
    return /[\*\?]/.test(item);
}
exports.containWildcards = containWildcards;
/**
 * Convert specific characters into regular expressions.
 */
function wildcardTransformer(wildcard, isPath) {
    // Add an escape character in front of special characters
    // special characters: '\', '^', '$', '.', '+', '|', '[', ']', '{', '}', '(', ')'
    let escapedItem = wildcard.replace(/[\\+^${}()|\[\]\.]/g, '\\$&');
    // isPath: containing '**', and '*', '?' can not be matched with '/'. 
    if (isPath) {
        // before: ../**/a/b/c*/?.ets
        // after: ../.*/a/b/c[^/]*/[^/].ets
        return escapedItem.replace(/\*\*/g, '.*').replace(/(?<!\.)\*/g, '[^/]*').replace(/\?/g, '[^/]');
    }
    // before: *a?
    // after: .*a.
    return escapedItem.replace(/\*/g, '.*').replace(/\?/g, '.');
}
exports.wildcardTransformer = wildcardTransformer;
/**
 * Determine whether the original name needs to be preserved.
 */
function needToBeReserved(reservedSet, universalArray, originalName) {
    return reservedSet.has(originalName) || isMatchWildcard(universalArray, originalName);
}
exports.needToBeReserved = needToBeReserved;
/**
 * Determine whether it can match the wildcard character in the array.
 */
function isMatchWildcard(wildcardArray, item) {
    for (const wildcard of wildcardArray) {
        if (wildcard.test(item)) {
            return true;
        }
    }
    return false;
}
exports.isMatchWildcard = isMatchWildcard;
/**
 * Separate parts of an array that contain wildcard characters.
 */
function handleReservedConfig(config, optionName, reservedListName, universalLisName, enableRemove) {
    const reservedConfig = config === null || config === void 0 ? void 0 : config[optionName];
    let needSeparate = !!(reservedConfig === null || reservedConfig === void 0 ? void 0 : reservedConfig[reservedListName]);
    if (enableRemove) {
        needSeparate && (needSeparate = reservedConfig[enableRemove]);
    }
    if (needSeparate) {
        // separate items which contain wildcards from others
        const reservedInfo = separateUniversalReservedItem(reservedConfig[reservedListName]);
        reservedConfig[reservedListName] = reservedInfo.specificReservedArray;
        reservedConfig[universalLisName] = reservedInfo.universalReservedArray;
    }
}
exports.handleReservedConfig = handleReservedConfig;
//# sourceMappingURL=TransformUtil.js.map